@@ -1,49 +1,27 @@
-from werkzeug.utils import import_string as wz_import_string
-from werkzeug.local import LocalProxy
+from werkzeug.utils import import_string as wz_import_string, cached_property
+from werkzeug.local import LocalProxy, LocalStack
 from flask import (url_for, Markup, json, request, _request_ctx_stack,\
-                   has_request_context)
+                   has_request_context, abort, current_app)
 import imp
 import functools
 import re
 from slugify import slugify
-import logging
 import yaml
 import speaklater
 import os
 from contextlib import contextmanager
-
-
-logger = logging.getLogger("frasco")
-# logger.setLevel(logging.DEBUG)
-# logger.addHandler(logging.StreamHandler())
-
-
-class RequirementMissingError(Exception):
-    pass
-
-
-class ClassBoundDecoratorMixin(object):
-    def _wrap_func(self, func):
-        @functools.wraps(func)
-        def bound_func(*args, **kwargs):
-            return self._call_func(func, *args, **kwargs)
-        bound_func.unbound_func = func
-        return bound_func
-
-    def _call_func(self, func, *args, **kwargs):
-        if not hasattr(self, "obj") or self.obj is None:
-            return func(*args, **kwargs)
-        return func(self.obj, *args, **kwargs)
-
-    def __get__(self, obj, cls):
-        self.obj = obj
-        return self
+import subprocess
+import click
+import inspect
+import logging
 
 
 def import_string(impstr, attr=None):
     """Imports a string. Can import an attribute of the imported
     class/module using a double colon as a separator
     """
+    if inspect.isclass(impstr):
+        return impstr
     if "::" in impstr:
         impstr, attr = impstr.split("::")
     imported = wz_import_string(impstr)
@@ -89,7 +67,7 @@ def find_classes_in_module(module, clstypes):
             for cls in clstypes:
                 if issubclass(item, cls) and item != cls:
                     classes.append(item)
-        except Exception as e:
+        except:
             pass
     return classes
 
@@ -127,12 +105,6 @@ def populate_obj(obj, attrs):
         setattr(obj, k, v)
 
 
-def copy_extra_feature_options(feature, target, prefix=""):
-    for k, v in feature.options.iteritems():
-        if not feature.defaults or k not in feature.defaults:
-            target["%s%s" % (prefix, k.upper())] = v
-
-
 def url_for_static(filename, **kwargs):
     """Shortcut function for url_for('static', filename=filename)
     """
@@ -151,20 +123,6 @@ def wrap_in_markup(f):
     return wrapper
 
 
-def deep_update_dict(a, b):
-    for k, v in b.iteritems():
-        if k not in a:
-            a[k] = v
-        elif isinstance(a[k], dict) and isinstance(v, dict):
-            deep_update_dict(a[k], v)
-        elif isinstance(a[k], list) and isinstance(v, list):
-            a[k].extend(v)
-        elif isinstance(v, list) and not isinstance(a[k], list):
-            a[k] = [a[k]] + v
-        else:
-            a[k] = v
-
-
 def make_kwarg_validator(name, validator_func):
     if not isinstance(name, tuple):
         name = (name,)
@@ -187,18 +145,46 @@ def kwarg_validator(name):
     return decorator
 
 
+def inject_app_config(app, config, prefix=None):
+    for k, v in config.iteritems():
+        if prefix:
+            k = "%s%s" % (prefix, k)
+        app.config[k.upper()] = v
+
+
+def extract_unmatched_items(items, allowed_keys, prefix=None, uppercase=False):
+    unmatched = {}
+    for k, v in items.iteritems():
+        if k not in allowed_keys:
+            if prefix:
+                k = "%s%s" % (prefix, k)
+            if uppercase:
+                k = k.upper()
+            unmatched[k] = v
+    return unmatched
+
+
+def deep_update_dict(a, b):
+    for k, v in b.iteritems():
+        if k not in a:
+            a[k] = v
+        elif isinstance(a[k], dict) and isinstance(v, dict):
+            deep_update_dict(a[k], v)
+        elif isinstance(a[k], list) and isinstance(v, list):
+            a[k].extend(v)
+        elif isinstance(v, list) and not isinstance(a[k], list):
+            a[k] = [a[k]] + v
+        else:
+            a[k] = v
+    return a
+
+
 class UnknownValue(object):
     pass
 
 unknown_value = UnknownValue()
 
 
-class DictObject(object):
-    def __init__(self, dct):
-        for k, v in dct.iteritems():
-            setattr(self, k, v)
-
-
 class AttrDict(dict):
     """Dict which keys are accessible as attributes
     """
@@ -217,6 +203,9 @@ class AttrDict(dict):
         except KeyError:
             raise KeyError(message)
 
+    def for_json(self):
+        return dict(self)
+
 
 class JSONEncoder(json.JSONEncoder):
     """A JSONEncoder which always activates the for_json feature
@@ -233,103 +222,55 @@ class JSONEncoder(json.JSONEncoder):
         return json.JSONEncoder.default(self, o)
 
 
-reloader_extra_dirs = ["actions", "features", "views", "services"]
+class ShellError(Exception):
+    def __init__(self, returncode, stderr):
+        super(ShellError, self).__init__(stderr)
+        self.returncode = returncode
+        self.stderr = stderr
 
 
-def get_reloader_extra_files(root='.'):
-    extra_files = []
-    for filename in os.listdir(root):
-        pathname = os.path.join(root, filename)
-        if os.path.isdir(pathname) and filename in reloader_extra_dirs:
-            for path, _, filenames in os.walk(pathname):
-                for f in filenames:
-                    if f.endswith('.html') or f.endswith('.yml'):
-                        extra_files.append(os.path.join(path, f))
-        elif filename.endswith('.yml'):
-            extra_files.append(pathname)
-    return extra_files
+def shell_exec(args, echo=True, fg="green", **kwargs):
+    kwargs["stdout"] = subprocess.PIPE
+    kwargs["stderr"] = subprocess.STDOUT
+    p = subprocess.Popen(args, **kwargs)
+    out, _ = p.communicate()
+    if p.returncode > 0:
+        raise ShellError(p.returncode, out)
+    if echo:
+        click.secho(out, fg=fg)
+    return out
 
 
-class ContextStack(object):
-    def __init__(self, top=None, default_item=None, allow_nested=True, ignore_nested=False):
-        self.stack = []
-        self.top = top
-        self.default_item = default_item
-        self.allow_nested = allow_nested
-        self.ignore_nested = ignore_nested
-
-    def push(self, item=unknown_value):
-        if self.stack and not self.allow_nested:
-            raise RuntimeError('Context does not support nesting')
-        if self.stack and self.ignore_nested:
-            item = self.top
-        elif item is unknown_value:
-            if callable(self.default_item):
-                item = self.default_item()
-            else:
-                item = self.default_item
-        self.stack.append(self.top)
-        self.top = item
-        return item
-
-    def pop(self):
-        top = self.top
-        self.top = self.stack.pop()
-        return top
-
-    @contextmanager
-    def ctx(self, item=unknown_value):
-        item = self.push(item)
-        try:
-            yield item
-        finally:
-            self.pop()
-
-    def __call__(self, *args, **kwargs):
-        return self.ctx(*args, **kwargs)
-
-    def make_proxy(self):
-        return LocalProxy(lambda: self.top)
-
-    def make_context_mixin(self):
-        class ContextMixin(object):
-            def __enter__(s):
-                self.push(s)
-                return s
-            def __exit__(s, exc_type, exc_val, exc_tb):
-                self.pop()
-                return False
-        return ContextMixin
+def get_remote_addr():
+    if current_app.debug and "__remoteaddr" in request.values:
+        return request.values["__remoteaddr"]
+    return request.remote_addr
 
 
-def context_stack_on_request_context(name, cls=ContextStack):
-    def _get_object():
-        if has_request_context() and not hasattr(_request_ctx_stack.top, name):
-            setattr(_request_ctx_stack.top, name, cls())
-        return getattr(_request_ctx_stack.top, name, None)
-    return LocalProxy(_get_object)
 
-
-class DelayedCallsContext(ContextStack):
-    def __init__(self):
-        super(DelayedCallsContext, self).__init__(default_item=list, ignore_nested=True)
-
-    def call(self, func, args, kwargs):
-        if self.top is not None:
-            self.top.append((func, args, kwargs))
-            return False
-        else:
-            func(*args, **kwargs)
+def match_domains(domain, allowed_domains):
+    for allowed_domain in allowed_domains:
+        if allowed_domain.startswith('^'):
+            if re.search(allowed_domain, domain, re.I):
+                return True
+        elif allowed_domain.lower() == domain.lower():
             return True
+    return False
 
-    def pop(self, drop_calls=False):
-        top = super(DelayedCallsContext, self).pop()
-        if not drop_calls and not self.stack:
-            for func, args, kwargs in top:
-                func(*args, **kwargs)
 
-    def proxy(self, func):
-        @functools.wraps(func)
-        def proxy(*args, **kwargs):
-            return self.call(func, args, kwargs)
-        return proxy
\ No newline at end of file
+def match_email_domain(email, allowed_domains):
+    if not email or not '@' in email:
+        return False
+    _, domain = email.split('@')
+    return match_domains(domain, allowed_domains)
+
+
+def remove_all_handlers_from_logger(logger):
+    for handler in logger.handlers:
+        logger.removeHandler(handler)
+    return logger
+
+
+class ExcludeWerkzeugLogFilter(logging.Filter):
+    def filter(self, record):
+        return record.name != 'werkzeug'
